import emoji

from pyrogram.types import Message
from pyrogram.enums import ChatType

from src.core.database import TgMessage, User, TgChat


def check_message_condition(
        message: Message,
) -> bool:
    if message.chat.type in [ChatType.PRIVATE, ChatType.BOT]:
        return False

    if not message.from_user:
        return False

    if not message.text:
        return False

    if message.from_user.username:
        if 'bot' in message.from_user.username.lower():
            return False

    emoji_count = emoji.emoji_count(str(message.text))
    if emoji_count > 3:
        return False

    return True



def create_and_format_message(
        messages: list[TgMessage],
        reason: str,
        chat: TgChat,
        user_model: User,
) -> str:
    def escape_markdown_v2(text: str) -> str:
        if not text:
            return ""
        escape_chars = '_*[]()~`>#+-=|{}.!'
        return ''.join('\\' + char if char in escape_chars else char for char in text)

    def validate_markdown(text: str) -> str:
        """Ensure all markdown entities are properly closed"""
        # Count backticks to ensure pairs
        backtick_count = text.count('`')
        if backtick_count % 3 != 0:  # Code blocks use triple backticks
            # Remove unpaired backticks
            text = text.replace('`', 'ʻ')  # Replace with similar-looking character
        return text

    # User info
    username = escape_markdown_v2(user_model.username) if user_model.username else f"ID: {user_model.id}"
    user_link = f"[{username}](tg://user?id={user_model.id})" if user_model.username else f"ID: {user_model.id}"

    # Chat info
    chat_title = escape_markdown_v2(chat.title)
    chat_link = (
        f"https://t.me/c/{str(chat.id)[4:]}"
        if str(chat.id).startswith("-100")
        else f"https://t.me/{chat.username}" if chat.username
        else f"Chat ID: {chat.id}"
    )

    # Escape reason
    reason_escaped = escape_markdown_v2(reason)

    # Header
    header = (
        f"🔥 *Найдена успешка!*\n"
        f"👤 *Пользователь:* {user_link}\n"
        f"🐩 *Чат:* [{chat_title}]({chat_link})\n"
        f"📌 *Причина:* {reason_escaped}\n\n"
        f"📝 *Диалог:*\n"
    )

    MAX_LENGTH = 4096
    header_length = len(header)
    available_length = MAX_LENGTH - header_length - 50  # Extra buffer for safety

    message_blocks = []
    current_length = 0

    for msg in messages:
        if not msg.text:
            continue

        # Clean and escape text
        clean_text = msg.text.replace('```', 'ʻʻʻ')  # Replace triple backticks
        escaped_text = escape_markdown_v2(clean_text)
        escaped_text = validate_markdown(escaped_text)

        sender_username = escape_markdown_v2(
            msg.user_relationship.username or f"ID:{msg.user_id}"
        )

        # Create message block with proper code block formatting
        block = (
            f"**{sender_username}:**\n"
            f"```\n"
            f"{escaped_text}\n"
            f"```\n\n"
        )
        block_length = len(block)

        if current_length + block_length > available_length:
            remaining_space = available_length - current_length
            if remaining_space > 30:  # Enough space for a truncated message
                truncated_text = escaped_text[:remaining_space - 30].rsplit(' ', 1)[0] + "..."
                block = (
                    f"**{sender_username}:**\n"
                    f"```\n"
                    f"{truncated_text}\n"
                    f"```\n\n"
                    f"_... сообщение обрезано ..._\n"
                )
                message_blocks.append(block)
            break

        message_blocks.append(block)
        current_length += block_length

    content = ''.join(message_blocks).strip()
    full_message = header + content

    # Final validation and truncation if needed
    full_message = validate_markdown(full_message)
    if len(full_message) > MAX_LENGTH:
        full_message = full_message[:MAX_LENGTH - 30].rsplit('\n', 2)[0] + "\n```\n...\n```\n_... сообщение обрезано ..._"

    return full_message